'use strict';
/**
 * Input Form
 * @typedef {Object} InputForm
 * @property {string} name - Name attribute
 * @property {string} type - Type attribute,
 * can be: button,checkbox,color,date,datetime-local,email,file,hidden,image,month,number,password,
 * radio,range,reset,search,submit,tel,text,time,url,week
 * @property {string} value - Value of the input
 * @property {string} placeholder
 * @property {boolean} checked
 * @property {string} innerHTML
 * @property {string} id - Important for the label
 * @property {string} label - Description/title for input
 */
/**
 * Convert Object to MySQL Compatible Strings
 * @param key
 * @param value
 * @returns {string}
 */
const convertToWHERE = function (key, value) {
	if (typeof value === 'number' || typeof value === 'boolean') {
		return key + ' = ' + value;
	} else if (typeof value === 'string') {
		return key + ' = "' + value + '"';
	} else {
		return '';
	}
};
/**
 * Lionel Client Object
 * @name LionelClient
 * @module
 */
const LionelClient = {
	/**
	 * @name call
	 * @param args
	 * @method
	 * @returns {*|Promise<any>}
	 */
	call: function (...args) {
		let i = 1;
		const isThereCallback = typeof args[args.length - 1] === 'function';
		const lastArgument = isThereCallback ? args.length - 1 : args.length;
		if (args.length === 0 || typeof args[0] !== 'string') {
			if (isThereCallback) {
				args[0](true, 'At least one parameters are needed for call');
			}
			return new Promise((resolve, reject) => {
				reject(new Error('At least one parameters are needed for call'));
			});
		}
		const functionName = args[0];
		const callback = isThereCallback ? args[lastArgument] : null;

		const params = [];
		for (i; i < lastArgument; i++) {
			params.push(args[i]);
		}

		return this._xmlHTTPRequest({
			callback: callback, // error result
			url: '/call',
			method: 'POST',
			'content-type': 'application/json',
			functionName: functionName,
			parameters: params,
		});
	},
	/**
	 * This function makes a Rest API Request to the server with the specific parameters to help server to recognize the client
	 * @param {Object} options
	 * @param {string} options.url
	 * @param {string} options.method
	 * @param {string} options.functionName
	 * @param {string[]} options.parameters
	 * @param {function|undefined} options.callback
	 * @returns {Promise<any>}
	 * @private
	 */
	_xmlHTTPRequest: function (options) {
		if (!options) options = {};
		const xHTTP = new XMLHttpRequest();
		const call = typeof options.url === 'string' ? options.url : '/call';
		const method = typeof options.method === 'string' ? options.method : 'POST';
		const content = typeof options['content-type'] === 'string' ? options['content-type'] : 'application/json';

		const forSend = {};
		forSend.method = options.functionName || '';
		forSend.arguments = options.parameters || [];
		const statusError = {
			message: 'You lost connection with the server',
			url: options.functionName
		};

		return new Promise((resolve, reject) => {
			xHTTP.onreadystatechange = function () {
				let error = true;
				if (this.readyState === 4 && this.status === 200) {
					error = false;
				} else if (this.readyState === 4) {
					if (typeof options.callback === 'function') {
						options.callback(true, statusError);
					}
					reject(statusError);
					return;
				}
				if (this.readyState === 4) {
					let response = xHTTP.response;
					if (response.charAt(0) === '{' || response.charAt(0) === '[') {
						try {
							response = JSON.parse(xHTTP.response);
						} catch (e) {
							console.error(e);
						}
					}
					if (error || (response && response.status && response.status === 'undefined')) {
						if (response.error === true) {
							error = true;
						}
						if (typeof options.callback === 'function') {
							options.callback(error, undefined);
						}
						reject(response);
					} else {
						if (typeof options.callback === 'function') {
							options.callback(error, response);
						}
						resolve(response);
					}
				}
			};

			xHTTP.open(method, call, true);
			xHTTP.setRequestHeader('Content-type', content);

			try {
				xHTTP.send(JSON.stringify(forSend));
			} catch (e) {
				if (typeof options.callback === 'function') {
					options.callback(true, statusError);
				}
				reject(statusError);
			}
		});
	},
	/**
	 * @name loadScript
	 * @method
	 * @description We can attack external scripts asynchronously to our page
	 * @param {string} url
	 * @param then - Callback [Optional]
	 * @returns {Promise<any>}
	 */
	loadScript: function (url, then) {
		return new Promise((resolve) => {
			const script = document.createElement('script');
			script.type = 'text/javascript';
			document.body.appendChild(script);

			script.onload = function () {
				if (typeof then === 'function') {
					then(script);
				}
				resolve(script);
			};
			script.src = url;
		});
	},
	/**
	 * @name getPublicJS
	 * @method
	 * @description We can load external javascript libraries through this Javascript function.
	 * This is a good async way the save network traffic
	 * @param {string} name
	 * @param {function|undefined} then
	 * @returns {Promise<any>}
	 */
	getPublicJS: function (name, then) {
		if (Array.isArray(name)) {
			name = name.join(';');
		} else if (typeof name === 'number') {
			name = name.toString();
		}

		return new Promise((resolve, reject) => {
			if (typeof name !== 'string') {
				reject(new Error('There is no name'));
			}
			let isDone = false;
			const n = name.split(';')[0];

			const callback = (error, result) => {
				if (!isDone) {
					isDone = true;
					if (typeof then === 'function') {
						then(result);
					}
					if (error) {
						reject(new Error(error));
					} else {
						resolve(result);
					}
				}
			};

			function _checkLib () {
				const node = document.querySelector('#externalLib' + n);
				if (node) {
					if (window[n]) {
						return window[n];
					} else {
						return _errorLib();
					}
				} else {
					return _errorLib();
				}
			}

			function _errorLib () {
				setTimeout(function () {
					if (!window[n]) {
						window[n] = function () {
							console.error('Libary is Not found: ' + name);
						};
					} else {
						callback(false, window[n]);
					}
				}, 1000);
				return null;
			}

			const createScript = function (name, html) {
				const script = document.createElement('script');
				script.type = 'text/javascript';
				script.id = 'externalLib' + name;
				script.innerHTML = html;
				// document.querySelector('body').appendChild(script);
				document.body.appendChild(script);
			};

			function _getLib () {
				LionelClient.call('__getPublicJS', name, function (error, result) {
					if (!error) {
						const node = document.querySelector('#externalLib' + n);
						if (!node) {
							if (name && name.indexOf(';') !== -1) {
								name.split(';').forEach(function (name, i) {
									console.log(name);
									if (!i) {
										createScript(name, result);
									} else {
										createScript(name, '//Placeholder, script is already loaded');
									}
								});
							} else {
								createScript(name, result);
							}
						}

						setTimeout(_statement, 30);
					}
				});
			}

			function _statement (getLib) {
				const lib = _checkLib();
				if (lib) {
					callback(false, lib);
				} else {
					if (typeof getLib === 'function') {
						getLib();
					}
				}
			}

			_statement(_getLib);
		});
	},
	callList: {},
	/**
	 * @name getPageContent
	 * @method
	 * @description You can use this function if you want to get HTML and JS data from a specific URL/file
	 * @param {String} name
	 * @returns {Promise<any>}
	 */
	getPageContent: function (name) {
		return new Promise((resolve, reject) => {
			if (!name) {
				resolve({});
			}
			if (name.indexOf('-') !== -1) {
				name = name.toString().split('-')[0];
			} else if (name.indexOf(':') !== -1) {
				name = name.toString().split(':')[0];
			}
			if (name.startsWith('/')) {
				name = name.substring(1);
			}
			// name = name.replace('/', '');
			LionelClient.call('__getRenderedTemplate', name, (error, result) => {
				if (error) {
					reject(new Error(error));
				} else {
					resolve(result);
				}
			});
		});
	},
	/**
	 * @name getPage
	 * @method
	 * @description Load pages/templates to Lionel
	 *  1. Get the template data from server {html:'HTML Code of page', onRendered:'JS code of page'}
	 *  2. Put HTML into &lt;div class='LionelPageContent'&gt;&lt;/div&gt; with LionelClient._renderPage
	 *  3. Checks that there is a &lt;script&gt; for this template or not.
	 *       -Yes: just call function from that window._onRendered_(nameofTemplate)()
	 *       -No:  Put into a &lt;script&gt; and append to body, after that will call!
	 * @param {String} name
	 * @param {string|undefined} parent
	 */
	getPage: function (name, parent) {
		this.clearAllIntervals();
		const self = this;
		if (parent === undefined) parent = '.LionelPageContent';

		if (name.includes('?')) { // Remove GET attributes
			name = name.split('?')[0];
		}
		if (name.startsWith('/')) {
			name = name.substring(1);
		}
		// name = name.replace('/', '');
		LionelClient.call('__getRenderedTemplate', name, function (error, result) {
			if (!error) {
				self._renderPage(name.replace('/', ''), parent, result);
			} else {
				if (result && result.details) {
					result = { html: 'Error: ' + result.details.message + '<br> URL: ' + result.details.url + '<br>Please refresh the page if it doesnt happen automatically.' };
					self._renderPage(name, parent, result);
					setTimeout(function () {
						location.reload();
					}, 5000);
				} else if (result && result.message) {
					result = { html: 'Error: ' + result.message + '<br> URL: ' + result.url + '<br>Please refresh the page if it doesnt happen automatically.' };
					self._renderPage(name, parent, result);
					setTimeout(function () {
						location.reload();
					}, 5000);
				}
			}
		}).catch(function (e) {
			if (e.url === '__getRenderedTemplate') {
				console.error('Session has no longer active or the server is down');
			}
		});
	},
	/**
	 * 1. Put HTML into &lt;div class='LionelPageContent'&gt;&lt;/div&gt; with LionelClient._renderPage
	 * 2. Checks that there is a &lt;script&gt; for this template or not.
	 *      -Yes: just call function from that global._onRendered_(nameofTemplate)()
	 *      -No:  Put into a &lt;script&gt; and append to body, after that will call!
	 *
	 * Handles some error from the server, but not too much
	 * @param {String} name      - Name of template
	 * @param {String} parent    - This usually .LionelPageContent
	 * @param {Object} result    - {html:'HTML Code of page', onRendered:'JS code of page'}
	 * @private
	 */
	_renderPage: function (name, parent, result) {
		if (document.querySelector('#__render_' + name) !== null) {
			document.querySelector('#__render_' + name).outerHTML = '';
		}
		if (result !== undefined && result !== false && typeof result === 'object') {
			const parentElement = document.querySelector(parent);
			if (parentElement === null) {
				document.body.innerHTML = '<h1>Bad Config on Client Side</h1>Browser will be refreshed in 3 seconds.';
				setTimeout(function () {
					location.reload();
				}, 3000);
				return;
			}
			parentElement.innerHTML = result.html;
			if (document.querySelector('#rendered' + name.replace(' ', '')) !== null && window['_onRendered_' + name] !== undefined) {
				LionelClient._pageOnRendered(name);
				LionelClient._scriptOnRendered('', '_onRendered_' + name);
			} else if (result.onRendered) {
				const script = document.createElement('script');
				script.type = 'text/javascript';
				script.id = 'rendered' + name.replace(' ', '');
				// if(global.devMode === true){
				//    script.src = 'console/'+name;
				// }else{
				const scriptName = !name.includes('"') ? '"_onRendered_' + name + '"' : '\'_onRendered_' + name + '\'';
				script.innerHTML = 'window[' + scriptName + ']=function(c){window.LionelError = "";LionelClient._pageOnRendered();try{\n' + result.onRendered + '\n}catch(e){LionelError = e;}if(typeof c === "function"){c(LionelError)};};LionelClient._scriptOnRendered(window.LionelError,"_onRendered_' + name + '");';
				// }
				// script.src = 'rendered/'+result.onRendered+'.js';
				document.querySelector('body').appendChild(script);
				// document.head.appendChild(script);
			}
		} else if (typeof result === 'string') {
			document.querySelector(parent).innerHTML = '<h1>Your session has expired</h1>The current page will be refreshed in 3 seconds.';
			setTimeout(function () {
				location.reload();
			}, 3000);
		}
	},
	_pageOnRendered: function () {
		this._runScriptsOnPage();
	},
	_scriptOnRendered: function (error, onRendered) {
		if (error) {
			console.warn('OnRendered unsuccessful: ' + error);
		}
		window[onRendered](function (error) {
			if (error) {
				console.error(error);
			}
		});
	},
	/**
	 * @name navigate
	 * @method
	 * @description This is an inner navigation call. You can navigate to another page without refreshing the browser.
	 * No need to download some external files and globals, so it saves network traffic
	 * @param href
	 */
	navigate: function (href) {
		if (href.charAt(0) === '/') {
			href = href.substring(1);
		}

		window.history.pushState({}, href, href); // TODO - navigating to links including colon (:) causes NS_ERROR_FAILURE

		href = href.replace('#', '');
		if (href === '') {
			LionelClient.getPage('index');
		} else {
			LionelClient.getPage(href.toString());
		}
	},
	/**
	 * Refresh scripts on a specific page
	 * @param parent
	 * @private
	 */
	_runScriptsOnPage: function (parent) {
		if (!parent) parent = '.LionelPageContent';
		document.querySelectorAll('body ' + parent + ' script').forEach(function (d) {
			const p = document.querySelector(parent);
			const src = d.getAttribute('src');
			const script = document.createElement('script');
			script.type = 'text/javascript';
			if (src !== null) {
				script.src = src;
			} else {
				script.innerHTML = d.innerHTML;
			}
			p.appendChild(script);
			d.outerHTML = '';
		});
	},
	intervalStorage: [],
	globals: {},
	/**
	 * @name clearAllIntervals
	 * @method
	 */
	clearAllIntervals: function () {
		const self = this;
		self.intervalStorage.forEach(function (d) {
			if (d) {
				self.clearInterval(d);
			}
		});
		self.intervalStorage = [];
	},
	/**
	 * @name setInterval
	 * @method
	 * @param {function} f
	 * @param {number} t
	 * @returns {number}
	 */
	setInterval: function (f, t) {
		const i = setInterval(f, t);
		this.intervalStorage.push(i);
		return i;
	},
	/**
	 * @name clearInterval
	 * @method
	 * @param {function} f
	 */
	clearInterval: function (f) {
		let i = null;
		if (this.intervalStorage) {
			this.intervalStorage.forEach(function (d, j) {
				if (d && d === f) {
					i = j;
				}
			});
		}
		if (i !== null) {
			delete this.intervalStorage[i];
		}
		return clearInterval(f);
	},
	/**
	 * @name setCookie
	 * @method
	 * @param {String} cookieName
	 * @param {String} cookieValue
	 */
	setCookie: function (cookieName, cookieValue) {
		const exdays = 1;
		const d = new Date();
		d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
		const expires = 'expires=' + d.toUTCString();
		document.cookie = cookieName + '=' + cookieValue + ';' + expires + ';path=/';
	},
	/**
	 * @name getCookie
	 * @method
	 * @param {String} cookieName
	 * @returns {string}
	 */
	getCookie: function (cookieName) {
		const name = cookieName + '=';
		const decodedCookie = decodeURIComponent(document.cookie);
		const ca = decodedCookie.split(';');
		for (let i = 0; i < ca.length; i++) {
			let c = ca[i];
			while (c.charAt(0) === ' ') {
				c = c.substring(1);
			}
			if (c.indexOf(name) === 0) {
				return c.substring(name.length, c.length);
			}
		}
		return '';
	},
	/**
	 * @name mongodb
	 * @method
	 * @param queryType
	 * @param query
	 * @returns {*|Promise<*>|{add: (function(*, *): *|Promise<*>), find: (function(*, *): *|Promise<*>), update: (function(*, *, *): *|Promise<*>), listCollections: (function(): *|Promise<*>), delete: (function(*, *): *|Promise<*>)}}
	 */
	mongodb: function (queryType, query) {
		if (queryType && query) {
			return LionelClient.call('mongodb', queryType, query);
		}
		return {
			listCollections: function () {
				return LionelClient.call('mongodb', 'list');
			},
			find: function (collection, filter) {
				const query = {
					collection: collection,
					object: filter
				};
				return LionelClient.call('mongodb', 'find', query);
			},
			update: function (collection, filter, details) {
				const query = {
					collection: collection,
					object: filter,
					details: details
				};
				return LionelClient.call('mongodb', 'update', query);
			},
			delete: function (collection, filter) {
				const query = {
					collection: collection,
					object: filter
				};
				return LionelClient.call('mongodb', 'delete', query);
			},
			add: function (collection, object) {
				const query = {
					collection: collection,
					object: object
				};
				return LionelClient.call('mongodb', 'add', query);
			}
		};
	},
	/**
	 * @name mysql
	 * @method
	 * @param queryType
	 * @param query
	 * @returns {*|Promise<*>|{listTables: (function(): *|Promise<*>), selectAll: (function(string): *|Promise<*>), select: ((function(string, string): (*|Promise<*>))|*), removeTable: (function(*): *|Promise<*>), update: ((function(string, (string|Object), (string|Object)): (*|Promise<*>))|*), delete: ((function(string, (string|Object)): (*|Promise<*>))|*)}}
	 */
	mysql: function (queryType, query) {
		if (queryType && query) {
			return LionelClient.call('mysql', queryType, query);
		}
		return {
			listTables: function () {
				return LionelClient.call('mysql', 'list');
			},
			removeTable: function (table) {
				return LionelClient.call('mysql', 'deleteTable', table);
			},
			/**
			 * @param {string} table
			 * @param {string} filter
			 */
			select: function (table, filter) {
				const target = {
					target: table,
					filter: filter
				};

				if (typeof target.filter === 'object') {
					const keys = Object.keys(target.filter);
					if (keys.length) {
						let where = '';
						keys.forEach(function (key, index) {
							if (index) {
								where += ' AND ';
							}
							where += convertToWHERE(key, target.filter[key]);
						});
						target.filter = where;
					} else {
						return LionelClient.call('mysql', 'listAllIn', table);
					}
				}

				return LionelClient.call('mysql', 'listIn', target);
			},
			/**
			 * @param {string} table
			 */
			selectAll: function (table) {
				return LionelClient.call('mysql', 'listAllIn', table);
			},
			/**
			 *
			 * @param {string} table
			 * @param {string|Object} filter
			 * @param {string|Object} data
			 * @returns {*|*|Promise<any>|Promise<any>}
			 */
			update: function (table, filter, data) {
				const query = {
					target: table,
					filter: filter,
					data: data
				};

				if (typeof query.filter === 'object') {
					const keys = Object.keys(query.filter);
					if (keys.length) {
						let where = '';
						keys.forEach(function (key, index) {
							if (index) {
								where += ' AND ';
							}
							where += convertToWHERE(key, query.filter[key]);
						});
						query.filter = where;
					} else {
						return new Promise((resolve, reject) => reject(new Error('Invalid Filter')));
					}
				}
				if (typeof query.data === 'object') {
					const keys = Object.keys(query.data);
					if (keys.length) {
						let where = '';
						keys.forEach(function (key, index) {
							if (index) {
								where += ', ';
							}
							where += convertToWHERE(key, query.data[key]);
						});
						query.data = where;
					} else {
						return new Promise((resolve, reject) => reject(new Error('Invalid Data')));
					}
				}
				return LionelClient.call('mysql', 'update', query);
			},
			/**
			 *
			 * @param {string} table
			 * @param {string|Object} filter
			 * @returns {*|*|Promise<any>|Promise<any>}
			 */
			delete: function (table, filter) {
				const query = {
					target: table,
					filter: filter,
				};

				if (typeof query.filter === 'object') {
					const keys = Object.keys(query.filter);
					if (keys.length) {
						let where = '';
						keys.forEach(function (key, index) {
							if (index) {
								where += ' AND ';
							}
							where += convertToWHERE(key, query.filter[key]);
						});
						query.filter = where;
					} else {
						return new Promise((resolve, reject) => reject(new Error('Invalid Filter')));
					}
				}
				return LionelClient.call('mysql', 'delete', query);
			},
		};
	},
	/**
	 * @name confirm
	 * @method
	 * @param {string} message - Message for Confirmation
	 * @param {undefined|string} title
	 * @returns {Promise<boolean>}
	 */
	confirm (message, title = 'Confirmation') {
		return new Promise(resolve => {
			const node = LionelClient.Helper.createPopup({
				title: title,
				innerHTML: message,
				onclose: () => {
					setTimeout(() => {
						resolve(false);
					}, 0);
				},
				buttons: [
					{
						value: 'OK',
						className: 'btn-primary',
						onclick: () => {
							node.outerHTML = '';
							resolve(true);
							return false;
						}
					},
					{
						value: 'Cancel',
						onclick: (e) => {
							node.outerHTML = '';
							resolve(false);
							return false;
						}
					}
				]
			}).node;
		});
	},
	/**
	 * @name alert
	 * @method
	 * @param {string} message - Message for Alert Box
	 * @param {undefined|string} title
	 * @returns {Promise<boolean>}
	 */
	alert (message, title = 'Message') {
		return new Promise(resolve => {
			const node = LionelClient.Helper.createPopup({
				title: title,
				innerHTML: message,
				onclose: () => {
					setTimeout(() => {
						resolve(false);
					}, 0);
				},
				buttons: [
					{
						value: 'OK',
						className: 'btn-primary',
						onclick: () => {
							node.outerHTML = '';
							resolve(true);
							return false;
						}
					}
				]
			}).node;
		});
	},
	/**
	 * @name LionelHelper
	 * @typedef LionelHelper
	 * @description Accessible via LionelClient.Helper
	 * @module
	 */
	Helper: {
		/**
		 * @name node
		 * @type {null|HTMLElement}
		 * @property
		 */
		node: null,
		/**
		 * @name detach
		 * @method
		 * @description Return a new LionelHelper object with the current edited node (original helper will be empty)
		 * @returns {LionelHelper}
		 */
		detach () {
			const node = this.node;
			this.node = null;
			const LionelHelper = Object.assign({}, LionelClient.Helper);
			LionelHelper.node = node;
			return LionelHelper;
		},
		/**
		 * @name getNode
		 * @method
		 * @returns {HTMLElement}
		 */
		getNode () {
			return this.node;
		},
		/**
		 * @name addClass
		 * @method
		 * @param {string|string[]} className
		 * @returns {LionelHelper}
		 */
		addClass (className) {
			const self = this;
			if (className && self.node instanceof HTMLElement) {
				if (typeof className === 'string') {
					className.split(' ').forEach(className => {
						if (className) {
							self.node.classList.add(className);
						}
					});
				} else if (Array.isArray(className)) {
					className.forEach(string => {
						if (typeof string === 'string' && string) {
							self.node.classList.add(string);
						}
					});
				}
			}
			return this;
		},

		/**
		 * @name setStyles
		 * @method
		 * @param {Object} styles
		 * @returns {LionelHelper}
		 */
		setStyles (styles) {
			const self = this;
			if (styles && typeof styles === 'object' && self.node instanceof HTMLElement) {
				const keys = Object.keys(styles);

				keys.forEach(key => {
					if (key && typeof key === 'string') {
						const parts = key.split('-');

						if (parts.length === 1) {
							self.node.style[key] = styles[key];
						} else {
							let formattedStyleName = '';
							parts.forEach((part, index) => {
								if (part) {
									if (index) {
										formattedStyleName += part.charAt(0) + part.substring(1);
									} else {
										formattedStyleName += part;
									}
								}
							});
							self.node.style[formattedStyleName] = styles[key];
						}
					}
				});
			}
			return this;
		},

		/**
		 * @name append
		 * @method
		 * @param nodes
		 * @returns {LionelHelper}
		 */
		append (...nodes) {
			if (this.node instanceof HTMLElement) {
				for (let i = 0; i < nodes.length; i++) {
					if (nodes[i] instanceof HTMLElement) {
						this.node.appendChild(nodes[i]);
					} else if (typeof nodes[i] === 'object' && nodes[i].node instanceof HTMLElement) {
						this.node.appendChild(nodes[i].node);
					}
				}
			}
			return this;
		},

		/**
		 * @name setAttributes
		 * @method
		 * @param {Object} attributes
		 * @returns {LionelHelper}
		 */
		setAttributes (attributes) {
			if (this.node instanceof HTMLElement && attributes && typeof attributes === 'object') {
				const keys = Object.keys(attributes);
				const self = this;

				keys.forEach(key => {
					self.node.setAttribute(key, attributes[key]);
				});
			}
			return this;
		},

		/**
		 * @name on
		 * @method
		 * @param {string} eventName
		 * @param {function} method
		 * @returns {LionelHelper}
		 */
		on (eventName, method) {
			if (this.node instanceof HTMLElement && eventName &&
				typeof eventName === 'string' && typeof method === 'function') {
				this.node['on' + eventName] = method;
			}
			return this;
		},

		/**
		 * @name setData
		 * @method
		 * @param {string} name
		 * @param {string} data
		 * @returns {LionelHelper}
		 */
		setData (name, data) {
			if (this.node instanceof HTMLElement && name && typeof name === 'string' && data) {
				this.node.setAttribute('data-' + name, data);
			}
			return this;
		},

		/**
		 * @name removeData
		 * @method
		 * @param {string} name
		 * @returns {LionelHelper}
		 */
		removeData (name) {
			if (this.node instanceof HTMLElement && name && typeof name === 'string') {
				this.node.removeAttribute('data-' + name);
			}
			return this;
		},

		/**
		 * @name getData
		 * @method
		 * @param {string} name
		 * @returns {string|null}
		 */
		getData (name) {
			if (this.node instanceof HTMLElement && name && typeof name === 'string') {
				return this.node.getAttribute(name);
			}
			return null;
		},

		/**
		 * @name getAllData
		 * @method
		 * @returns {object[]}
		 */
		getAllData () {
			const attributes = this.attributes;
			const length = attributes.length;
			const result = [];
			for (let i = 0; i < length; i++) {
				result.push({
					name: attributes[i].name,
					value: attributes[i].value
				});
			}
			return result;
		},

		/**
		 * @name createElement
		 * @method
		 * @param {string} type
		 * @param {Object} details
		 * @param details.className
		 * @param details.classList
		 * @param details.styles
		 * @param details.innerHTML
		 * @param details.innerText
		 * @param details.innerElement
		 * @param details.innerElements
		 * @param details.id
		 * @param details.checked
		 * @param details.attributes
		 * @returns {LionelHelper}
		 */
		createElement (type, details) {
			if (!type) {
				type = 'div';
			}
			if (!details) {
				details = {};
			}
			if (typeof type !== 'string' || typeof details !== 'object') {
				console.error('Invalid type/details');
				this.node = null;
				return this;
			}
			this.node = document.createElement(type);
			const self = this;

			if (details.className) {
				this.addClass(details.className);
			}
			if (details.classList) {
				this.addClass(details.classList);
			}
			if (details.styles) {
				this.setStyles(details);
			}
			if (details.innerHTML) {
				this.node.innerHTML = details.innerHTML;
			}
			if (details.innerText) {
				this.node.innerHTML = details.innerText;
			}
			if (details.innerElement instanceof HTMLElement) {
				this.node.appendChild(details.innerElement);
			}
			if (details.attributes) {
				this.setAttributes(details.attributes);
			}
			if (Array.isArray(details.innerElements)) {
				details.innerElements.forEach(element => {
					if (element instanceof HTMLElement) {
						self.node.appendChild(element);
					}
				});
			}
			if (details.id) {
				this.node.id = details.id;
			}
			if (details.checked) {
				this.node.checked = details.checked;
			}
			return this;
		},

		/**
		 * @name createTable
		 * @method
		 * @param {Array[]} lineArray
		 * @param {Object} details
		 * @param details.className
		 * @param details.classList
		 * @param details.styles
		 * @param details.header
		 * @param details.footer
		 * @param details.events
		 * @returns {LionelHelper}
		 */
		createTable (lineArray, details) {
			if (!lineArray) {
				lineArray = [];
			}

			if (!details) {
				details = {};
			}
			if (!Array.isArray(lineArray) || typeof details !== 'object') {
				console.error('Invalid type/details');
				this.node = null;
				return this;
			}

			this.node = document.createElement('table');

			if (details.className) {
				this.addClass(details.className);
			}
			if (details.classList) {
				this.addClass(details.classList);
			}
			if (details.styles) {
				this.setStyles(details);
			}

			const getTableLine = function (columns, type = 'td') {
				const tr = document.createElement('tr');

				if (Array.isArray(columns)) {
					columns.forEach(column => {
						const tableElement = document.createElement(type);
						if (typeof column === 'string' || typeof column === 'number') {
							tableElement.innerHTML = column;
						} else if (typeof column === 'function') {
							const functionResult = column();

							if (functionResult instanceof HTMLElement) {
								tableElement.appendChild(functionResult);
							} else if (functionResult) {
								tableElement.innerHTML = functionResult;
							}
						} else if (column instanceof HTMLElement) {
							tableElement.appendChild(column);
						} else if (column && typeof column === 'object') {
							try {
								tableElement.innerHTML = JSON.stringify(column);
							} catch (e) {
								console.warn(e);
							}
						} else {
							console.warn('Invalid column data');
						}

						tr.appendChild(tableElement);
					});
				}

				if (details.events && typeof details.events === 'object') {
					const names = Object.keys(details.events);
					names.forEach(event => {
						if (typeof details.events[event] === 'function') {
							tr[event] = function (ev) {
								if (typeof ev.preventDefault === 'function') {
									ev.preventDefault();
								}
								details.events[event](columns, ev);
							};
						}
					});
				}

				return tr;
			};

			const thead = document.createElement('thead');
			if (Array.isArray(details.header)) {
				thead.appendChild(getTableLine(details.header, 'th'));
			}
			this.node.appendChild(thead);
			const tbody = document.createElement('tbody');
			lineArray.forEach(line => {
				tbody.appendChild(getTableLine(line));
			});
			this.node.appendChild(tbody);

			if (Array.isArray(details.footer)) {
				const tfoot = document.createElement('tfoot');
				tfoot.appendChild(getTableLine(details.footer));
				this.node.appendChild(tfoot);
			}
			return this;
		},

		/**
		 * @name createSelector
		 * @method
		 * @param {Array} options
		 * @param {Object} details
		 * @param details.className
		 * @param details.classList
		 * @param details.styles
		 * @param details.label
		 * @param details.id
		 * @returns {LionelHelper}
		 */
		createSelector (options, details) {
			if (!options) {
				options = [];
			}

			if (!details) {
				details = {};
			}
			if (!Array.isArray(options) || typeof details !== 'object') {
				console.error('Invalid type/details');
				this.node = null;
				return this;
			}

			this.node = document.createElement('select');
			const self = this;
			if (details.className) {
				this.addClass(details.className);
			}
			if (details.classList) {
				this.addClass(details.classList);
			}
			if (details.styles) {
				this.setStyles(details);
			}
			if (details.label) {
				this.node.label = details.label;
			}
			if (details.id) {
				this.node.label = details.id;
			}

			options.forEach(option => {
				if (option) {
					const node = document.createElement('option');

					if (typeof option === 'string') {
						node.setAttribute('value', option);
						node.innerHTML = option;
					} else if (typeof option === 'object') {
						node.innerHTML = option.name || option.innerHTML || '';
						node.setAttribute('value', option.value || '');
					}
					self.node.appendChild(node);
				}
			});
			return this;
		},

		/**
		 * @name select
		 * @method
		 * @param {string|HTMLElement} selector
		 * @returns {LionelHelper}
		 */
		select (selector) {
			if (selector) {
				const self = this;

				if (typeof selector === 'string') {
					self.node = document.querySelector(selector);
				} else if (selector instanceof HTMLElement) {
					self.node = selector;
				}
			}
			return this;
		},

		/**
		 * @name createForm
		 * @method
		 * @param {InputForm[]} inputs
		 * @param {Object} details
		 * @param details.title
		 * @param details.action
		 * @param details.method
		 * @param details.target
		 * @param details.className
		 * @param details.classList
		 * @param details.styles
		 * @returns {LionelHelper}
		 */
		createForm (inputs, details) {
			if (!inputs) {
				inputs = [];
			}

			if (!details) {
				details = {};
			}
			if (!inputs || typeof details !== 'object') {
				console.error('Invalid type/details');
				this.node = null;
				return this;
			}

			this.node = document.createElement('form');
			const self = this;

			if (details.className) {
				this.addClass(details.className);
			}
			if (details.classList) {
				this.addClass(details.classList);
			}
			if (details.styles) {
				this.setStyles(details);
			}

			const createInput = input => {
				if (input) {
					if (input instanceof HTMLElement) {
						if (input.label) {
							const label = document.createElement('label');
							if (input.id) {
								label.setAttribute('for', input.id);
							}
							label.innerHTML = input.label;
							self.node.appendChild(label);
						}
						self.node.appendChild(input);
					} else if (typeof input === 'object') {
						const node = document.createElement('input');
						node.setAttribute('type', input.type || 'text');
						if (input.value) {
							node.setAttribute('value', input.value);
						}
						if (typeof input.checked === 'boolean') {
							node.setAttribute('value', input.checked);
						}
						if (input.placeholder) {
							node.setAttribute('value', input.placeholder);
						}
						if (input.name) {
							node.setAttribute('value', input.name);
						}
						if (input.innerHTML) {
							node.innerHTML = input.innerHTML;
						}
						if (input.className) {
							node.className = input.className;
						}
						if (input.id) {
							node.id = input.id;
						}
						if (input.label) {
							const label = document.createElement('label');
							if (input.id) {
								label.setAttribute('for', input.id);
							}
							label.innerHTML = input.label;
							self.node.appendChild(label);
						}
						self.node.appendChild(node);
					}
				}
			};

			if (details.title) {
				const title = document.createElement('h5');
				title.classList.add('title');
				title.innerHTML = details.title;
				this.node.appendChild(title);
			}
			if (Array.isArray(inputs)) {
				inputs.forEach(createInput);
			} else if (typeof inputs === 'object') {
				const inputList = Object.keys(inputs);
				inputList.forEach(id => {
					inputs[id].id = id;
					createInput(inputs[id]);
				});
			} else {
				console.warn('Invalid input');
			}

			if (details.action) {
				this.node.setAttribute('action', details.action);
			}
			if (details.method) {
				this.node.setAttribute('method', details.method);
			}
			if (details.target) {
				this.node.setAttribute('target', details.target);
			}

			return this;
		},

		/**
		 * @name createWindow
		 * @method
		 * @param {Object} options
		 * @param options.title
		 * @param options.onclose
		 * @param options.nodes
		 * @param options.innerHTML
		 * @param options.buttons
		 * @returns {LionelClient}
		 */
		createWindow (options) {
			if (!options) {
				options = {};
			}
			this.node = document.createElement('div');
			this.addClass('modal-content');
			this.setStyles({
				margin: '5px',
				width: 'auto'
			});

			const header = document.createElement('div');
			header.classList.add('modal-header');

			const title = document.createElement('h5');
			title.innerHTML = options.title || '';

			const closeButton = document.createElement('button');
			closeButton.classList.add('close');
			closeButton.setAttribute('type', 'button');
			closeButton.style.zIndex = '999';

			const times = document.createElement('span');
			times.setAttribute('aria-hidden', 'true');
			times.innerHTML = '×';

			if (typeof options.onclose === 'function') {
				closeButton.onclick = function () {
					if (self.node.parentNode) {
						self.node.outerHTML = '';
					}
					options.onclose();
				};
			} else if (options.onclose === 'destroy') {
				const self = this;
				closeButton.onclick = function () {
					if (self.node.parentNode) {
						self.node.outerHTML = '';
					}
				};
			}
			closeButton.appendChild(times);

			header.appendChild(title);
			header.appendChild(closeButton);

			this.node.appendChild(header);

			const content = document.createElement('div');
			content.classList.add('modal-body');

			const self = this;
			if (Array.isArray(options.nodes)) {
				options.nodes.forEach(node => {
					if (node instanceof HTMLElement) {
						content.appendChild(node);
					}
				});
			}
			if (typeof options.innerHTML === 'string' || typeof options.innerHTML === 'number') {
				content.innerHTML += options.innerHTML;
			}

			self.node.appendChild(content);

			const footer = document.createElement('div');
			footer.classList.add('modal-footer');

			if (Array.isArray(options.buttons)) {
				options.buttons.forEach(button => {
					const element = document.createElement('button');
					element.classList.add('btn');
					element.setAttribute('type', 'button');

					if (button && typeof button === 'object') {
						if (typeof button.className === 'string') {
							element.className += ' ' + button.className;
						}
						if (typeof button.onclick === 'function') {
							element.onclick = function (e) {
								if (button.onclick.call(self, e)) {
									self.node.outerHTML = '';
								}
							};
						} else {
							element.onclick = function () {
								self.node.outerHTML = '';
							};
						}
						if (typeof button.innerHTML === 'string') {
							element.innerHTML = button.innerHTML;
						}
						if (typeof button.value === 'string') {
							element.innerHTML = button.value;
						}
						footer.appendChild(element);
					}
				});
				self.node.appendChild(footer);
			}

			return this;
		},

		/**
		 * @name createPopup
		 * @method
		 * @param {Object} options
		 * @param {string} options.title
		 * @param {undefined|Node[]|Element[]}options.nodes
		 * @param {undefined|string} options.innerHTML
		 * @param {undefined|object[]} options.buttons
		 * @param {undefined|function} options.apply
		 * @param {undefined|function} options.onclose
		 * @param {undefined|'sm'|'lg'|'xl'} options.containerSize
		 * @returns {LionelClient}
		 */
		createPopup (options) {
			if (!options) {
				options = {};
			}

			const background = document.createElement('div');
			background.className = 'modal fade show';
			background.style.display = 'block';
			background.style.paddingRight = '17px';
			background.style.background = '#8888888c';
			background.setAttribute('role', 'dialog');

			let closeCalled = false;
			if (typeof options.onclose === 'function') {
				const _onclose = options.onclose;
				options.onclose = function () {
					if (!closeCalled) {
						closeCalled = true;
						_onclose();
					}
					if (background.parentNode) {
						background.outerHTML = '';
					}
				};
			} else {
				options.onclose = function () {
					if (!closeCalled) {
						closeCalled = true;
					}
					if (background.parentNode) {
						background.outerHTML = '';
					}
				};
			}
			background.onclick = function (e) {
				if (e && e.target instanceof HTMLElement) {
					if (e.target.className && e.target.className === 'modal fade show') {
						if (!closeCalled) {
							closeCalled = true;
							options.onclose();
						}
					} else if (e.target.className === 'close' ||
						(e.target.tagName.toUpperCase() === 'SPAN' && e.target.parentElement.className === 'close')) {
						if (!closeCalled) {
							closeCalled = true;
							options.onclose();
						}
					}
				}
			};

			const dialog = document.createElement('div');
			dialog.classList.add('modal-dialog');
			if (options.containerSize) {
				dialog.classList.add('modal-' + options.containerSize);
			}
			dialog.setAttribute('role', 'document');

			dialog.appendChild(this.createWindow(options).node);

			background.appendChild(dialog);

			this.node = background;
			if (options.apply instanceof HTMLElement) {
				options.apply.appendChild(background);
			} else {
				document.body.appendChild(background);
			}
			return this;
		},

		/**
		 * @name createNavHeader
		 * @method
		 * @param {function} options.isActive
		 * @param {boolean} options.vertical
		 * @param {string} options.id
		 * @param {Array} options.items
		 * @param {string} options.default
		 * @param {string} options.alignment
		 * @param {string} options.colorScheme
		 * @param {string} options.background
		 * @param {string} options.icon
		 * @param {string} options.title
		 * @param {string} options.bootstrap
		 * @param {'fluid'|'lg'|'sm'|'md'|'xl'|'xxl'|undefined|'default'} options.container
		 * @param options
		 * @returns {LionelClient}
		 */
		createNavHeader (options) {
			if (!options) {
				options = {};
			}
			if (!options.bootstrap) {
				options.bootstrap = '4';
			}
			const navOuter = document.createElement('nav');
			navOuter.classList.add('navbar');
			navOuter.classList.add('navbar-expand-lg');
			if (options.vertical) {
				navOuter.classList.add('flex-column');
			}

			if (options.colorScheme === 'dark') {
				navOuter.classList.add('navbar-dark');
				if (typeof options.background === 'string') {
					navOuter.style.backgroundColor = options.background;
				} else {
					navOuter.classList.add('bg-dark');
				}
			} else {
				navOuter.classList.add('navbar-light');
				if (typeof options.background === 'string') {
					navOuter.style.backgroundColor = options.background;
				} else {
					navOuter.classList.add('bg-light');
				}
			}
			let navFluidContainer;
			if (options.bootstrap === '5' || options.container) {
				navFluidContainer = document.createElement('div');
				if (options.container) {
					options.container = options.container.trim();
				}
				if ((options.bootstrap === '5' && options.container) || options.container === 'fluid') {
					navFluidContainer.classList.add('container-' + options.container);
				} else {
					navFluidContainer.classList.add('container');
				}
			} else {
				navFluidContainer = navOuter;
			}

			const brand = document.createElement('a');
			brand.classList.add('navbar-brand');
			brand.href = '#';
			if (options.icon) {
				const img = document.createElement('img');
				img.src = options.icon;
				img.width = 30;
				img.height = 30;
				if (options.title) {
					img.classList.add('d-inline-block');
					img.classList.add('align-top');
					brand.appendChild(img);
					brand.innerHTML += ' ' + options.title;
				} else {
					brand.appendChild(img);
				}
				navFluidContainer.appendChild(brand);
			} else if (options.title) {
				brand.innerHTML += options.title;
				navFluidContainer.appendChild(brand);
			}

			const navbar = document.createElement('div');
			if (!options.vertical) {
				const customInnerId = 'nav' + new Date().getTime();
				navbar.classList.add('collapse');
				navbar.classList.add('navbar-collapse');
				navbar.id = customInnerId;

				const toggleButton = document.createElement('button');
				toggleButton.className = 'navbar-toggler';
				toggleButton.setAttribute('type', 'button');
				toggleButton.setAttribute('data-toggle', 'collapse');
				toggleButton.setAttribute('data-target', '#' + customInnerId);
				toggleButton.setAttribute('aria-controls', 'navbarSupportedContent');
				toggleButton.setAttribute('aria-expanded', 'false');
				toggleButton.setAttribute('aria-label', 'Toggle navigatiopn');

				toggleButton.innerHTML = '<span class="navbar-toggler-icon"></span>';

				navFluidContainer.appendChild(toggleButton);
			}

			this.createNavigation(options);
			const navigation = this.node;
			navbar.appendChild(navigation);
			navFluidContainer.appendChild(navbar);
			if (options.bootstrap === '5' || options.container) {
				navOuter.appendChild(navFluidContainer);
				this.node = navOuter;
			} else {
				this.node = navFluidContainer;
			}

			return this;
		},

		/**
		 * @name GETParameters
		 * @method
		 * @returns {object}
		 */
		GETParameters () {
			if (!location.search.startsWith('?')) {
				return {};
			}
			const parameters = location.search.substring(1) || '';
			const output = {};
			const gets = parameters.split('&');
			gets.forEach(function (keyValue) {
				if (keyValue) {
					const pair = keyValue.split('=');
					if (pair.length === 2) {
						const number = parseInt(pair[1]);
						if (!Number.isNaN(number)) {
							output[pair[0]] = number;
						} else if (pair[1] === 'true') {
							output[pair[0]] = true;
						} else if (pair[1] === 'false') {
							output[pair[0]] = false;
						} else if (pair[1].startsWith('{') || pair[1].startsWith('[')) {
							try {
								output[pair[0]] = JSON.parse(decodeURI(pair[1]));
							} catch (e) {
								output[pair[0]] = pair[1];
							}
						} else if (!pair[1]) {
							output[pair[0]] = null;
						} else {
							output[pair[0]] = pair[1];
						}
					}
				}
			});
			return output;
		},

		/**
		 * @name createNavigation
		 * @method
		 * @param options
		 * @param {function} options.isActive
		 * @param {boolean} options.vertical
		 * @param {string} options.id
		 * @param {Array} options.items
		 * @param {string} options.default
		 * @param {string} options.alignment
		 * @param {string} options.bootstrap
		 * @param {boolean|undefined} options.caseInSensitive
		 * @returns {LionelClient}
		 */
		createNavigation (options) {
			if (!options) {
				options = {};
			}
			this.node = document.createElement('ul');
			if (!options.id) {
				options.id = 'navID' + new Date().getTime();
			}
			if (!options.bootstrap) {
				options.bootstrap = '4';
			}
			if (typeof options.caseInSensitive !== 'boolean') {
				options.caseInSensitive = true;
			}
			this.node.id = options.id;
			const possibleAlignments = ['left', 'center', 'right'];
			const alignment = options.alignment && possibleAlignments.includes(options.alignment) ? options.alignment : 'left';
			if (options.vertical) {
				this.addClass('navbar-nav flex-column');
			} else {
				if (alignment === 'center') {
					this.setStyles({
						marginLeft: 'auto',
						marginRight: 'auto'
					});
					this.addClass('navbar-nav');
				} else if (alignment === 'left') {
					if (options.bootstrap === '5') {
						this.addClass('navbar-nav me-auto');
					} else {
						this.addClass('navbar-nav mr-auto');
					}
				} else if (alignment === 'right') {
					if (options.bootstrap === '5') {
						this.addClass('navbar-nav ms-auto');
					} else {
						this.addClass('navbar-nav ml-auto');
					}
				}
			}
			if (Array.isArray(options.items)) {
				const isDefault = function (item, index, pathName) {
					if (pathName === '/') {
						if (options.default === item || (options.caseInSensitive && options.default.toLowerCase() === item.toLowerCase())) {
							return true;
						} else if (!options.default && !index) {
							return true;
						}
					}
					return false;
				};
				const onClickFunction = function (item, index, pathName) {
					if (options.caseInSensitive) {
						LionelClient.navigate('/' + item.toLowerCase());
					} else {
						LionelClient.navigate('/' + item);
					}

					document.querySelectorAll('#' + options.id + ' li.nav-item > a').forEach((a, index) => {
						if ((a.innerHTML === item) || (options.caseInSensitive && a.innerHTML.toLowerCase() === item.toLowerCase()) ||
							(pathName === '/' && !index)) {
							a.parentElement.classList.add('active');
						} else {
							a.parentElement.classList.remove('active');
						}
					});
				};
				const pathName = decodeURI(location.pathname);
				options.items.forEach((item, index) => {
					if (item) {
						const li = document.createElement('li');
						li.classList.add('nav-item');
						const a = document.createElement('a');
						a.classList.add('nav-link');
						if (typeof item === 'string') {
							// a.setAttribute('href','');
							a.innerHTML = item;
							if ((pathName.startsWith('/') && pathName.substring(1) === item) ||
								isDefault(item, index, pathName)) {
								li.classList.add('active');
							}
							a.onclick = function () {
								onClickFunction(item, index, pathName);
							};
						} else if (item instanceof HTMLElement) {
							li.appendChild(item);
							this.node.appendChild(li);
							return;
						} else if (typeof item === 'object') {
							if (item.href) {
								a.setAttribute('href', item.href);
							}
							if (typeof item.onclick === 'function') {
								a.onclick = item.onclick;
							}
							if (typeof item.icon === 'string') {
								a.innerHTML = '<img src="' + item.icon + '"  alt=""/>';
							} else {
								a.innerHTML = '';
							}
							a.innerHTML += item.innerHTML;
							if (typeof options.isActive === 'function' && options.isActive(li, item.innerHTML, item, pathName)) {
								li.classList.add('active');
							} else if (item.href) {
								if (pathName === item.href || pathName === item.href.split('?')[0].split('#')[0] ||
									isDefault(item.href.substring(1), index, pathName)) {
									li.classList.add('active');
								}
							}
						}
						li.appendChild(a);
						this.node.appendChild(li);
					}
				});
			}
			return this;
		},

		/**
		 * @name createSidebar
		 * @method
		 * @param options
		 * @param {function} options.isActive
		 * @param {boolean} options.vertical
		 * @param {string} options.id
		 * @param {Array} options.items
		 * @param {string} options.default
		 * @param {string} options.parent
		 * @param {string} options.colorScheme
		 * @param {'left'|'right'} options.position
		 * @param options.styles
		 * @returns {HTMLElement}
		 */
		createSidebar (options) {
			if (typeof options !== 'object' || !options) {
				options = {};
			}
			const sidebar = LionelClient.Helper.createElement('aside', {});

			if (options.colorScheme === 'dark') {
				sidebar.node.classList.add('navbar-dark');
				sidebar.node.classList.add('bg-dark');
			} else {
				sidebar.node.classList.add('navbar-light');
				sidebar.node.classList.add('bg-light');
			}
			sidebar.node.style.padding = '5px 20px';
			if (options.styles) {
				LionelClient.Helper.setStyles(options.styles);
			}
			if (!options.vertical) {
				options.vertical = true;
			}
			const sidebarElement = sidebar.node;
			const navigation = LionelClient.Helper.createNavigation(options).node;
			sidebarElement.appendChild(navigation);

			if (options.position) {
				const pathName = decodeURI(location.pathname.substring(1).split('?')[0]);
				if (!options.items.includes(pathName)) {
					if (!options.parent || typeof options.parent !== 'string' || pathName !== options.parent) {
						return sidebarElement;
					}
				}
				if (typeof options.parent === 'string') {
					document.querySelectorAll('header.main .nav-link').forEach(function (node) {
						if (options.parent === node.innerHTML) {
							node.parentElement.classList.add('active');
						} else {
							node.parentElement.classList.remove('active');
						}
					});
				}
				const container = document.querySelector('.LionelPageContent');
				if (container) {
					container.style.display = 'flex';
					container.style.alignItems = 'stretch';
					if (options.position === 'left') {
						const mainInner = container.childNodes[0];
						container.insertBefore(sidebarElement, mainInner);
					}
					if (options.position === 'right') {
						container.appendChild(sidebarElement);
					}
				}
			}
			return sidebarElement;
		}
	},
};
if (!window._modules) {
	window._modules = {};
}
module.exports = { LionelClient };
